@@ -17,6 +17,7 @@ Settings module
 
 Provides few constants, APIs endpoints.
 """
+import os
 from os import getenv
 
 
@@ -44,7 +45,15 @@ class Settings:
             "Get measurements of for host": "/api/atlas/v1.0/groups/{GROUP-ID}/processes/{HOST}:{PORT}/disks/{"
                                             "DISK-NAME}/measurements",
             "Get the log file for a host in the cluster": "/api/atlas/v1.0/groups/{group_id}/clusters/{"
-                                                          "host}/logs/{logname}"
+                                                          "host}/logs/{logname}",
+            "Get Available Disks for Process": "/api/atlas/v1.0/groups/{group_id}/processes/"
+                                               "{host}:{port}/disks",
+            "Get Measurements of a Disk for Process": "/api/atlas/v1.0/groups/{group_id}/processes/{host}:{port}/disks/"
+                                                      "{disk_name}/measurements",
+            "Get Measurements of a Database for Process": "/api/atlas/v1.0/groups/{group_id}/processes/{host}:{port}/"
+                                                          "databases/{database_name}/measurements",
+            "Get Available Databases for Process": "/api/atlas/v1.0/groups/{group_id}/processes/"
+                                                   "{host}:{port}/databases"
         },
         "Events": {
             "Get All Project Events": URI_STUB + "/groups/{group_id}/events??includeRaw=true&pageNum={page_num}"
@@ -144,7 +153,7 @@ class Settings:
             "Orgs the authenticated user can access": URI_STUB + "/orgs/",
             "Org by org_id": URI_STUB + "/orgs/{ORG_ID}",
             "Atlas Users associated to Org": URI_STUB + "/orgs/{ORGS_ID}/users/",
-            "Projects associated with the Org": URI_STUB + "/groups/{ORG_ID}/groups"
+            "Projects associated with the Org": URI_STUB + "/orgs/{ORG_ID}/groups"
         }
 
     }
@@ -155,9 +164,9 @@ class Settings:
 
     # Atlas default pagination
     pageNum = 1
-    itemsPerPage = 1000
-    itemsPerPageMin = 1
-    itemsPerPageMax = 2000
+    itemsPerPage: int = int(os.getenv('ITEMS_PER_PAGE', 500))
+    itemsPerPageMin: int = int(os.getenv('ITEMS_PER_PAGE_MIN', 1))
+    itemsPerPageMax: int = int(os.getenv('ITEMS_PER_PAGE_MAX', 2000))
 
     # Requests
     requests_timeout = 10
